@@ -70,7 +70,7 @@ static bool parse_and(const bson_t *in, bson_iter_t *out, mongocrypt_status_t *s
     BSON_ASSERT_PARAM(out);
     BSON_ASSERT(status || true);
 
-    bson_iter_t and;
+    bson_iter_t and = {0};
     if (!bson_iter_init(&and, in) || !bson_iter_next(&and) || 0 != strcmp(bson_iter_key(&and), "$and")) {
         ERR_WITH_BSON(in, "%s", "error unable to find '$and'");
         return false;
@@ -105,7 +105,7 @@ parse_aggregate_expression(const bson_t *orig, bson_iter_t *in, operator_value_t
     BSON_ASSERT_PARAM(out);
     BSON_ASSERT(status || true);
 
-    bson_iter_t array, value;
+    bson_iter_t array = {0}, value;
     const char *op_type_str = bson_iter_key(in);
     bool ok = false;
     const char *field;
@@ -162,7 +162,7 @@ parse_match_expression(const bson_t *orig, bson_iter_t *in, operator_value_t *ou
     BSON_ASSERT_PARAM(out);
     BSON_ASSERT(status || true);
 
-    bson_iter_t document, value;
+    bson_iter_t document = {0}, value;
     const char *op_type_str;
     bool ok = false;
     const char *field = bson_iter_key(in);
@@ -217,7 +217,7 @@ bool mc_FLE2RangeFindDriverSpec_parse(mc_FLE2RangeFindDriverSpec_t *spec,
     // {$and: [{$gt: ["$age", 5]}, {$lt:["$age", 50]}]}
     // Or `in` may be a Match Expression with this form:
     // {$and: [{age: {$gt: 5}}, {age: {$lt: 50}} ]}
-    bson_iter_t and, array;
+    bson_iter_t and = {0}, array = {0};
     bool ok = false;
 
     if (!parse_and(in, &and, status)) {
@@ -264,7 +264,7 @@ bool mc_FLE2RangeFindDriverSpec_parse(mc_FLE2RangeFindDriverSpec_t *spec,
             }
         }
 
-        operator_value_t op;
+        operator_value_t op = {0};
         switch (arg_type) {
         case AGGREGATE_EXPRESSION:
             if (!parse_aggregate_expression(in, &doc, &op, status)) {
@@ -367,6 +367,10 @@ bool mc_makeRangeFindPlaceholder(mc_makeRangeFindPlaceholder_args_t *args,
             BSON_ASSERT(args->precision.value <= INT32_MAX);
             TRY(BSON_APPEND_INT32(edgesInfo, "precision", (int32_t)args->precision.value));
         }
+        if (args->trimFactor.set) {
+            BSON_ASSERT(args->trimFactor.value <= INT32_MAX);
+            TRY(BSON_APPEND_INT32(edgesInfo, "trimFactor", (int32_t)args->trimFactor.value));
+        }
         TRY(BSON_APPEND_DOCUMENT(v, "edgesInfo", edgesInfo));
     }
 
@@ -383,7 +387,7 @@ bool mc_makeRangeFindPlaceholder(mc_makeRangeFindPlaceholder_args_t *args,
     TRY(_mongocrypt_buffer_append(args->index_key_id, p, "ki", 2));
     TRY(_mongocrypt_buffer_append(args->user_key_id, p, "ku", 2));
     TRY(BSON_APPEND_DOCUMENT(p, "v", v));
-    TRY(BSON_APPEND_INT64(p, "cm", args->maxContentionCounter));
+    TRY(BSON_APPEND_INT64(p, "cm", args->maxContentionFactor));
     TRY(BSON_APPEND_INT64(p, "s", args->sparsity));
 #undef TRY
 
@@ -403,7 +407,7 @@ fail:
 
 bool mc_FLE2RangeFindDriverSpec_to_placeholders(mc_FLE2RangeFindDriverSpec_t *spec,
                                                 const mc_RangeOpts_t *range_opts,
-                                                int64_t maxContentionCounter,
+                                                int64_t maxContentionFactor,
                                                 const _mongocrypt_buffer_t *user_key_id,
                                                 const _mongocrypt_buffer_t *index_key_id,
                                                 int32_t payloadId,
@@ -469,8 +473,9 @@ bool mc_FLE2RangeFindDriverSpec_to_placeholders(mc_FLE2RangeFindDriverSpec_t *sp
                                                .indexMin = indexMin,
                                                .indexMax = indexMax,
                                                .precision = range_opts->precision,
-                                               .maxContentionCounter = maxContentionCounter,
-                                               .sparsity = range_opts->sparsity};
+                                               .maxContentionFactor = maxContentionFactor,
+                                               .sparsity = range_opts->sparsity,
+                                               .trimFactor = range_opts->trimFactor};
 
     // First operator is the non-stub.
     if (!mc_makeRangeFindPlaceholder(&args, &p1, status)) {
@@ -485,7 +490,7 @@ bool mc_FLE2RangeFindDriverSpec_to_placeholders(mc_FLE2RangeFindDriverSpec_t *sp
                                                    .payloadId = payloadId,
                                                    .firstOp = spec->firstOp,
                                                    .secondOp = spec->secondOp,
-                                                   .maxContentionCounter = maxContentionCounter,
+                                                   .maxContentionFactor = maxContentionFactor,
                                                    .sparsity = range_opts->sparsity};
 
         // First operator is the non-stub.
